/*
*
* Space for BeOS R5
* - Release 2.0
*
* Copyright (C) 1999-2000 Brian Matzon. All Rights Reserved.
*
* This program is freeware.
*
* Please send any comments and/or flames, ideas ... to brian@matzon.dk
* 
*
*/

#include "SpaceView.h"

//Set all member vars to what was specified in the constructor.
SpaceView::SpaceView(int w, int h, rgb_color& Active, rgb_color& Idle, rgb_color& Frame)
: BView(BRect(0, 0, w, h), "SpaceView", B_FOLLOW_LEFT | B_FOLLOW_TOP, B_WILL_DRAW | B_PULSE_NEEDED), ActiveColor(Active), IdleColor(Idle), FrameColor(Frame)
{
}

//Extract any information that has been added to the BMessage 
//object in the Archive function.
SpaceView::SpaceView(BMessage* Msg)
: BView(Msg)
{
	rgb_color* Rgb;
	ssize_t Size;
	status_t Status;
	
	Status = Msg->FindData("FrameColor", B_RGB_COLOR_TYPE, (const void **)&Rgb, &Size);
	FrameColor = Status == B_NO_ERROR ? *Rgb : D_FrameColor;
	Status = Msg->FindData("ActiveColor", B_RGB_COLOR_TYPE, (const void **)&Rgb, &Size);
	ActiveColor = Status == B_NO_ERROR ? *Rgb : D_ActiveColor;
	Status = Msg->FindData("IdleColor", B_RGB_COLOR_TYPE, (const void **)&Rgb, &Size);
	IdleColor = Status == B_NO_ERROR ? *Rgb : D_IdleColor;
}

//Destructor...
SpaceView::~SpaceView()
{}

//Add any information that is needed to instantiate a new object from 
//a BMessage. Could be colors and stuff which was parsed from the cmd. line
//and then passed to the std. constructor. 
status_t SpaceView::Archive(BMessage* Msg, bool Deep) const
{
	//Let the Super class archive the information it needs to archive
	status_t Error = BView::Archive(Msg, Deep);
	
	//We need to add the Application signature of the program so that
	//the Deskbar can find it's (i think) code/datasegment.
	Msg->AddString("add_on", "application/x-vnd.Matzon.Space");

	//Add the Member variables to the BMessage, since this view and all it's data will be wiped in a couple 'o' secs
	//And we need to use the "options" specified by the user (doh!)
	//The ability to specify these colors will be added later, when time permits it...
	Msg->AddData("FrameColor", B_RGB_COLOR_TYPE, &FrameColor, sizeof(rgb_color));
	Msg->AddData("ActiveColor", B_RGB_COLOR_TYPE, &ActiveColor, sizeof(rgb_color));
	Msg->AddData("IdleColor", B_RGB_COLOR_TYPE, &IdleColor, sizeof(rgb_color));
	
	return Error;
}

//This is were we return an instance of the class. we also do a sanity check!
BArchivable* SpaceView::Instantiate(BMessage* Msg)
{
	if(!validate_instantiation(Msg, "SpaceView"))
		return NULL;

	//All ok. return a BArchivable object... (The BView which SpaceView
	//inherits from in turn inherits from BArchivable soo...)
	return (new SpaceView(Msg));
}

void SpaceView::AttachedToWindow()
{
	SetDrawingMode(B_OP_COPY);
	SetViewColor(0, 0, 0);
	Roster.GetBootVolume(&Volume);
	
	//We need to set the rate at which we want update's. Since SpaceView
	//Monitors the harddrive space there's really no need for fast updates
	//so we only update every second.
	BMessenger messenger(this);
	messagerunner = new BMessageRunner(messenger, new BMessage(PULSE_SPACE), 1000000, -1);
}

//Removed from Deskbar...
void SpaceView::DetachedFromWindow()
{}

//The method called every second.
void SpaceView::Pulse()
{}

//This is the draw function... do what it takes... :)
void SpaceView::Draw(BRect UpdateRect)
{
	BRect rect(Bounds());
	Occupied = (int)(rect.Height() - 2) - 14 * Volume.FreeBytes()/Volume.Capacity();
	
	//Draw border
	SetHighColor(FrameColor);
	StrokeRect(BRect(rect.left - 1, rect.top - 1, rect.right + 1, rect.bottom + 1));	
	
	//Draw unoccupied space
	SetHighColor(IdleColor);
	BRect NewRect(rect.left + 1, rect.top + 1 , rect.right - 1, (rect.bottom - 1) - Occupied);
	FillRect(NewRect);

	//Draw occupied space
	SetHighColor(ActiveColor);
	BRect secNewRect(rect.left + 1, rect.bottom - Occupied, rect.right - 1, rect.bottom - 1);
	FillRect(secNewRect);

	//Draw Fraction
	float t = (rect.Height() - 2.0) - 14.0 * Volume.FreeBytes()/Volume.Capacity();
	rgb_color FractionColor;
	FractionColor.red = FractionColor.green = 0;
	FractionColor.blue = (uint8)255.0 + (uint8)(255.0 * (float)((float)t - (float)Occupied));
	SetHighColor(FractionColor);
	StrokeLine(BPoint(rect.left+1, (rect.bottom) - Occupied), BPoint(rect.right-1, (rect.bottom) - Occupied)); 
}

//This is the MouseDown hook which get's called when someone presses the mouse
//over the view.
void SpaceView::MouseDown(BPoint Where)
{
	uint32 Button = 0;
	GetMouse(&Where, &Button);
	if(Button == B_SECONDARY_MOUSE_BUTTON) 
	{
		//Right click... build the Popupmenu. Set targets. Find location of mouse. Show it. Catch selected item, and
		//send it's message to its target.
		BMenuItem* Selected;
		BMenuItem* mniRemove  = new BMenuItem("Remove", new BMessage(REMOVE_SPACE));
		BMenuItem* mniAbout  = new BMenuItem("About", new BMessage(B_ABOUT_REQUESTED));

		mniRemove->SetTarget(this);
		mniAbout->SetTarget(this);

		ConvertToScreen(&Where);

		BPopUpMenu* Pop = new BPopUpMenu("SpaceMenu", false);
		Pop->AddItem(mniRemove);
		Pop->AddSeparatorItem();
		Pop->AddItem(mniAbout);
		Pop->SetFont(be_plain_font);
		Pop->SetFontSize(10.0);
		Selected = Pop->Go(Where);
		if (Selected)
		{
			BLooper *Looper;
			BHandler *Target = Selected->Target(&Looper);
			Looper->PostMessage(Selected->Message(), Target);
		}
		delete Pop;
	}
}

//This is the incomming mail. :)
//Sort it and handle it.
void SpaceView::MessageReceived(BMessage* Msg)
{
	switch(Msg->what)
	{
		case B_ABOUT_REQUESTED:
		{
			//Show some info...
			(new BAlert("About Space", "    Space - The Replicant\n\n Programming and stuff by\n\n           Brian Matzon\n       brian@matzon.dk","Whatever..."))->Go(NULL);
			break;
		}
		case REMOVE_SPACE:
		{
			BDeskbar *deskbar = new BDeskbar();
			deskbar->RemoveItem("SpaceView");
			delete deskbar;
		}
		case PULSE_SPACE:
		{
			Draw(Bounds());
		}
		default:
		BView::MessageReceived(Msg);
	}
}